# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_Private_Modules/04_save_result.ipynb.

# %% auto 0
__all__ = ['SaveResult']

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 2
# Copyright © 2023-2024  IfcTruss Contributors
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 4
from fastcore.basics import patch
import ifcopenshell
import numpy as np
import pandas as pd

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 14
class SaveResult:
    def __init__(
        self,
        *,
        model: ifcopenshell.file,
        structural_analysis_model,
        load_group,
        theory_type,
        is_linear: bool,
        displacments,
        forces,
        normal_forces
    ):
        self.ifc_model = model
        self.displacment = displacments
        self.force = forces
        self.normal_force = normal_forces

        self.structural_analysis_model = self.ifc_model.by_guid(
            structural_analysis_model
        )
        self.load_group = self.ifc_model.by_guid(load_group)
        self.theory_type = theory_type
        self.is_linear = is_linear

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 16
@patch
def create_structural_result_group(
    self: SaveResult,
):
    self.structural_result_group = (
        self.ifc_model.createIfcStructuralResultGroup(
            ifcopenshell.guid.new(),
            None,
            None,
            None,
            None,
            self.theory_type,
            self.load_group,
            self.is_linear,
        )
    )

    if self.structural_analysis_model.HasResults is not None:
        self.structural_analysis_model.HasResults += (
            self.structural_result_group,
        )
    elif self.structural_analysis_model.HasResults is None:
        self.structural_analysis_model.HasResults = (
            self.structural_result_group,
        )

    self.ifc_results = []

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 22
@patch
def create_displacment_for_one_node(self: SaveResult, row):
    node = row["Node"]
    displacement_x = row["Displacement_X"]
    displacement_y = row["Displacement_Y"]
    displacement_z = row["Displacement_Z"]

    name = "Result_Displacement"
    structural_load_single_displacement = (
        self.ifc_model.createIfcStructuralLoadSingleDisplacement(
            name,
            displacement_x,
            displacement_y,
            displacement_z,
            None,
            None,
            None,
        )
    )

    global_or_local = "GLOBAL_COORDS"
    structural_point_reaction = (
        self.ifc_model.createIfcStructuralPointReaction(
            ifcopenshell.guid.new(),
            None,
            name,
            None,
            None,
            None,
            None,
            structural_load_single_displacement,
            global_or_local,
        )
    )

    self.ifc_model.createIfcRelConnectsStructuralActivity(
        ifcopenshell.guid.new(),
        None,
        None,
        None,
        self.ifc_model.by_guid(node),
        structural_point_reaction,
    )

    return structural_point_reaction

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 23
@patch
def create_displacments(
    self: SaveResult,
):
    self.ifc_results += (
        self.displacment.apply(self.create_displacment_for_one_node, axis=1)
    ).tolist()

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 29
@patch
def create_force_for_one_node(self: SaveResult, row):
    node = row["Node"]
    force_x = row["Force_X"]
    force_y = row["Force_Y"]
    force_z = row["Force_Z"]
    name = "Result_Force"
    structural_load_single_force = (
        self.ifc_model.createIfcStructuralLoadSingleForce(
            name, force_x, force_y, force_z, None, None, None
        )
    )

    global_or_local = "GLOBAL_COORDS"

    structural_point_reaction = (
        self.ifc_model.createIfcStructuralPointReaction(
            ifcopenshell.guid.new(),
            None,
            name,
            None,
            None,
            None,
            None,
            structural_load_single_force,
            global_or_local,
        )
    )

    self.ifc_model.createIfcRelConnectsStructuralActivity(
        ifcopenshell.guid.new(),
        None,
        None,
        None,
        self.ifc_model.by_guid(node),
        structural_point_reaction,
    )

    return structural_point_reaction

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 30
@patch
def create_forces(
    self: SaveResult,
):
    self.ifc_results += (
        self.force.apply(self.create_force_for_one_node, axis=1)
    ).tolist()

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 37
@patch
def calculate_differences(self: SaveResult, x_1, y_1, z_1, x_2, y_2, z_2):
    # copied from DirectStiffnessMethod class
    l_x = x_2 - x_1
    l_y = y_2 - y_1
    l_z = z_2 - z_1
    return l_x, l_y, l_z

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 39
@patch
def calculate_distance(self: SaveResult, l_x, l_y, l_z):
    # copied from DirectStiffnessMethod class
    vector = np.array([l_x, l_y, l_z])
    l = np.linalg.norm(vector)
    return l

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 40
@patch
def get_coordinates(self: SaveResult, node):
    # copied from View class and modified
    coordinates = node.VertexGeometry.Coordinates
    return coordinates

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 41
@patch
def get_start_node(self: SaveResult, bar):
    # copied from View class and modified
    start_node = bar.Representation.Representations[0].Items[0].EdgeStart
    return start_node

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 42
@patch
def get_end_node(self: SaveResult, bar):
    # copied from View class and modified
    end_node = bar.Representation.Representations[0].Items[0].EdgeEnd
    return end_node

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 45
@patch
def get_length(self: SaveResult, row):
    bar = row["Bar"]
    normal_force = row["Normal_force"]

    ifc_bar = self.ifc_model.by_guid(bar)

    start_node = self.get_start_node(ifc_bar)
    end_node = self.get_end_node(ifc_bar)

    start_node_coordinates = self.get_coordinates(start_node)
    end_node_coordinates = self.get_coordinates(end_node)

    l_x, l_y, l_z = self.calculate_differences(
        *start_node_coordinates, *end_node_coordinates
    )

    length = self.calculate_distance(l_x, l_y, l_z)

    return length

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 46
@patch
def create_normal_force_for_one_bar(self: SaveResult, row):
    bar = row["Bar"]
    normal_force = row["Normal_force"]
    name = "Result_Normal_Force"
    length = self.get_length(row)

    structural_load_single_force_tail = (
        self.ifc_model.createIfcStructuralLoadSingleForce(
            Name="Tail",
            ForceX=normal_force,
        )
    )

    structural_load_single_force_head = (
        self.ifc_model.createIfcStructuralLoadSingleForce(
            Name="Head",
            ForceX=-normal_force,
        )
    )

    structural_load_configuration = (
        self.ifc_model.createIfcStructuralLoadConfiguration(
            Name="Member End Reactions",
            Values=(
                structural_load_single_force_head,
                structural_load_single_force_tail,
            ),
            Locations=[
                [0.0],
                [float(length)],
            ],
        )
    )
    global_or_local = "LOCAL_COORDS"
    predefined_type = "DISCRETE"
    structural_curve_reaction = (
        self.ifc_model.createIfcStructuralCurveReaction(
            ifcopenshell.guid.new(),
            None,
            name,
            None,
            None,
            None,
            None,
            structural_load_configuration,
            global_or_local,
            predefined_type,
        )
    )

    self.ifc_model.createIfcRelConnectsStructuralActivity(
        ifcopenshell.guid.new(),
        None,
        None,
        None,
        self.ifc_model.by_guid(bar),
        structural_curve_reaction,
    )

    return structural_curve_reaction

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 47
@patch
def create_normal_forces(
    self: SaveResult,
):
    self.ifc_results += (
        self.normal_force.apply(
            self.create_normal_force_for_one_bar, axis=1
        )
    ).tolist()

# %% ../nbs/03_Private_Modules/04_save_result.ipynb 53
@patch
def assign_to_the_result_group(
    self: SaveResult,
):
    self.ifc_model.createIfcRelAssignsToGroup(
        ifcopenshell.guid.new(),
        None,
        None,
        None,
        self.ifc_results,
        None,
        self.structural_result_group,
    )
